@@ -4,7 +4,7 @@ import typing as t
 from enum import auto
 
 from sqlglot.helper import AutoName
-from sqlglot.trie import in_trie, new_trie
+from sqlglot.trie import TrieResult, in_trie, new_trie
 
 
 class TokenType(AutoName):
@@ -137,6 +137,7 @@ class TokenType(AutoName):
     BIGSERIAL = auto()
     XML = auto()
     UNIQUEIDENTIFIER = auto()
+    USERDEFINED = auto()
     MONEY = auto()
     SMALLMONEY = auto()
     ROWVERSION = auto()
@@ -144,6 +145,7 @@ class TokenType(AutoName):
     VARIANT = auto()
     OBJECT = auto()
     INET = auto()
+    ENUM = auto()
 
     # keywords
     ALIAS = auto()
@@ -162,6 +164,7 @@ class TokenType(AutoName):
     CACHE = auto()
     CASE = auto()
     CHARACTER_SET = auto()
+    CLUSTER_BY = auto()
     COLLATE = auto()
     COMMAND = auto()
     COMMENT = auto()
@@ -181,6 +184,7 @@ class TokenType(AutoName):
     DESCRIBE = auto()
     DICTIONARY = auto()
     DISTINCT = auto()
+    DISTRIBUTE_BY = auto()
     DIV = auto()
     DROP = auto()
     ELSE = auto()
@@ -195,6 +199,7 @@ class TokenType(AutoName):
     FINAL = auto()
     FIRST = auto()
     FOR = auto()
+    FORCE = auto()
     FOREIGN_KEY = auto()
     FORMAT = auto()
     FROM = auto()
@@ -207,6 +212,7 @@ class TokenType(AutoName):
     HAVING = auto()
     HINT = auto()
     IF = auto()
+    IGNORE = auto()
     ILIKE = auto()
     ILIKE_ANY = auto()
     IN = auto()
@@ -281,6 +287,7 @@ class TokenType(AutoName):
     SHOW = auto()
     SIMILAR_TO = auto()
     SOME = auto()
+    SORT_BY = auto()
     STRUCT = auto()
     TABLE_SAMPLE = auto()
     TEMPORARY = auto()
@@ -346,6 +353,7 @@ class Token:
             col: The column that the token ends on.
             start: The start index of the token.
             end: The ending index of the token.
+            comments: The comments to attach to the token.
         """
         self.token_type = token_type
         self.text = text
@@ -391,12 +399,15 @@ class _Tokenizer(type):
 
         klass._STRING_ESCAPES = set(klass.STRING_ESCAPES)
         klass._IDENTIFIER_ESCAPES = set(klass.IDENTIFIER_ESCAPES)
-        klass._COMMENTS = dict(
-            (comment, None) if isinstance(comment, str) else (comment[0], comment[1])
-            for comment in klass.COMMENTS
-        )
+        klass._COMMENTS = {
+            **dict(
+                (comment, None) if isinstance(comment, str) else (comment[0], comment[1])
+                for comment in klass.COMMENTS
+            ),
+            "{#": "#}",  # Ensure Jinja comments are tokenized correctly in all dialects
+        }
 
-        klass.KEYWORD_TRIE = new_trie(
+        klass._KEYWORD_TRIE = new_trie(
             key.upper()
             for key in (
                 *klass.KEYWORDS,
@@ -456,20 +467,22 @@ class Tokenizer(metaclass=_Tokenizer):
     STRING_ESCAPES = ["'"]
     VAR_SINGLE_TOKENS: t.Set[str] = set()
 
+    # Autofilled
+    IDENTIFIERS_CAN_START_WITH_DIGIT: bool = False
+
     _COMMENTS: t.Dict[str, str] = {}
     _FORMAT_STRINGS: t.Dict[str, t.Tuple[str, TokenType]] = {}
     _IDENTIFIERS: t.Dict[str, str] = {}
     _IDENTIFIER_ESCAPES: t.Set[str] = set()
     _QUOTES: t.Dict[str, str] = {}
     _STRING_ESCAPES: t.Set[str] = set()
+    _KEYWORD_TRIE: t.Dict = {}
 
-    KEYWORDS: t.Dict[t.Optional[str], TokenType] = {
+    KEYWORDS: t.Dict[str, TokenType] = {
         **{f"{{%{postfix}": TokenType.BLOCK_START for postfix in ("", "+", "-")},
         **{f"{prefix}%}}": TokenType.BLOCK_END for prefix in ("", "+", "-")},
-        "{{+": TokenType.BLOCK_START,
-        "{{-": TokenType.BLOCK_START,
-        "+}}": TokenType.BLOCK_END,
-        "-}}": TokenType.BLOCK_END,
+        **{f"{{{{{postfix}": TokenType.BLOCK_START for postfix in ("+", "-")},
+        **{f"{prefix}}}}}": TokenType.BLOCK_END for prefix in ("+", "-")},
         "/*+": TokenType.HINT,
         "==": TokenType.EQ,
         "::": TokenType.DCOLON,
@@ -502,6 +515,7 @@ class Tokenizer(metaclass=_Tokenizer):
         "UNCACHE": TokenType.UNCACHE,
         "CASE": TokenType.CASE,
         "CHARACTER SET": TokenType.CHARACTER_SET,
+        "CLUSTER BY": TokenType.CLUSTER_BY,
         "COLLATE": TokenType.COLLATE,
         "COLUMN": TokenType.COLUMN,
         "COMMIT": TokenType.COMMIT,
@@ -519,6 +533,7 @@ class Tokenizer(metaclass=_Tokenizer):
         "DESC": TokenType.DESC,
         "DESCRIBE": TokenType.DESCRIBE,
         "DISTINCT": TokenType.DISTINCT,
+        "DISTRIBUTE BY": TokenType.DISTRIBUTE_BY,
         "DIV": TokenType.DIV,
         "DROP": TokenType.DROP,
         "ELSE": TokenType.ELSE,
@@ -594,6 +609,7 @@ class Tokenizer(metaclass=_Tokenizer):
         "RECURSIVE": TokenType.RECURSIVE,
         "REGEXP": TokenType.RLIKE,
         "REPLACE": TokenType.REPLACE,
+        "RETURNING": TokenType.RETURNING,
         "REFERENCES": TokenType.REFERENCES,
         "RIGHT": TokenType.RIGHT,
         "RLIKE": TokenType.RLIKE,
@@ -609,6 +625,7 @@ class Tokenizer(metaclass=_Tokenizer):
         "SHOW": TokenType.SHOW,
         "SIMILAR TO": TokenType.SIMILAR_TO,
         "SOME": TokenType.SOME,
+        "SORT BY": TokenType.SORT_BY,
         "TABLE": TokenType.TABLE,
         "TABLESAMPLE": TokenType.TABLE_SAMPLE,
         "TEMP": TokenType.TEMPORARY,
@@ -709,6 +726,7 @@ class Tokenizer(metaclass=_Tokenizer):
         "PREPARE": TokenType.COMMAND,
         "TRUNCATE": TokenType.COMMAND,
         "VACUUM": TokenType.COMMAND,
+        "USER-DEFINED": TokenType.USERDEFINED,
     }
 
     WHITE_SPACE: t.Dict[t.Optional[str], TokenType] = {
@@ -732,8 +750,7 @@ class Tokenizer(metaclass=_Tokenizer):
     NUMERIC_LITERALS: t.Dict[str, str] = {}
     ENCODE: t.Optional[str] = None
 
-    COMMENTS = ["--", ("/*", "*/"), ("{#", "#}")]
-    KEYWORD_TRIE: t.Dict = {}  # autofilled
+    COMMENTS = ["--", ("/*", "*/")]
 
     __slots__ = (
         "sql",
@@ -748,7 +765,6 @@ class Tokenizer(metaclass=_Tokenizer):
         "_end",
         "_peek",
         "_prev_token_line",
-        "identifiers_can_start_with_digit",
     )
 
     def __init__(self) -> None:
@@ -894,18 +910,18 @@ class Tokenizer(metaclass=_Tokenizer):
         char = chars
         prev_space = False
         skip = False
-        trie = self.KEYWORD_TRIE
+        trie = self._KEYWORD_TRIE
         single_token = char in self.SINGLE_TOKENS
 
         while chars:
             if skip:
-                result = 1
+                result = TrieResult.PREFIX
             else:
                 result, trie = in_trie(trie, char.upper())
 
-            if result == 0:
+            if result == TrieResult.FAILED:
                 break
-            if result == 2:
+            if result == TrieResult.EXISTS:
                 word = chars
 
             size += 1
@@ -994,7 +1010,7 @@ class Tokenizer(metaclass=_Tokenizer):
                 self._advance()
             elif self._peek == "." and not decimal:
                 after = self.peek(1)
-                if after.isdigit() or not after.strip():
+                if after.isdigit() or not after.isalpha():
                     decimal = True
                     self._advance()
                 else:
@@ -1013,13 +1029,13 @@ class Tokenizer(metaclass=_Tokenizer):
                     literal += self._peek.upper()
                     self._advance()
 
-                token_type = self.KEYWORDS.get(self.NUMERIC_LITERALS.get(literal))
+                token_type = self.KEYWORDS.get(self.NUMERIC_LITERALS.get(literal, ""))
 
                 if token_type:
                     self._add(TokenType.NUMBER, number_text)
                     self._add(TokenType.DCOLON, "::")
                     return self._add(token_type, literal)
-                elif self.identifiers_can_start_with_digit:  # type: ignore
+                elif self.IDENTIFIERS_CAN_START_WITH_DIGIT:
                     return self._add(TokenType.VAR)
 
                 self._add(TokenType.NUMBER, number_text)
