@@ -485,7 +485,7 @@ class TestDialect(Validator):
                 "duckdb": "CAST(x AS DATE)",
                 "hive": "TO_DATE(x)",
                 "postgres": "CAST(x AS DATE)",
-                "presto": "CAST(SUBSTR(CAST(x AS VARCHAR), 1, 10) AS DATE)",
+                "presto": "CAST(CAST(x AS TIMESTAMP) AS DATE)",
                 "snowflake": "CAST(x AS DATE)",
             },
         )
@@ -749,14 +749,14 @@ class TestDialect(Validator):
                 "drill": "DATE_ADD(CAST('2021-02-01' AS DATE), INTERVAL 1 DAY)",
                 "duckdb": "CAST('2021-02-01' AS DATE) + INTERVAL 1 DAY",
                 "hive": "DATE_ADD('2021-02-01', 1)",
-                "presto": "DATE_ADD('DAY', 1, DATE_PARSE(SUBSTR('2021-02-01', 1, 10), '%Y-%m-%d'))",
+                "presto": "DATE_ADD('DAY', 1, CAST(CAST('2021-02-01' AS TIMESTAMP) AS DATE))",
                 "spark": "DATE_ADD('2021-02-01', 1)",
             },
         )
         self.validate_all(
             "TS_OR_DS_ADD(x, 1, 'DAY')",
             write={
-                "presto": "DATE_ADD('DAY', 1, DATE_PARSE(SUBSTR(CAST(x AS VARCHAR), 1, 10), '%Y-%m-%d'))",
+                "presto": "DATE_ADD('DAY', 1, CAST(CAST(x AS TIMESTAMP) AS DATE))",
                 "hive": "DATE_ADD(x, 1)",
             },
         )
@@ -1084,6 +1084,14 @@ class TestDialect(Validator):
         self.validate_identity("some.column LIKE 'foo' + another.column + 'bar'")
 
         self.validate_all("LIKE(x, 'z')", write={"": "'z' LIKE x"})
+        self.validate_all(
+            "CONCAT(a, b, c)",
+            write={
+                "": "CONCAT(a, b, c)",
+                "redshift": "a || b || c",
+                "sqlite": "a || b || c",
+            },
+        )
         self.validate_all(
             "x ILIKE '%y'",
             read={
@@ -1177,10 +1185,21 @@ class TestDialect(Validator):
         self.validate_all(
             "CONCAT(a)",
             write={
-                "mysql": "a",
+                "clickhouse": "a",
+                "presto": "a",
+                "trino": "a",
                 "tsql": "a",
             },
         )
+        self.validate_all(
+            "COALESCE(CAST(a AS TEXT), '')",
+            read={
+                "drill": "CONCAT(a)",
+                "duckdb": "CONCAT(a)",
+                "postgres": "CONCAT(a)",
+                "tsql": "CONCAT(a)",
+            },
+        )
         self.validate_all(
             "IF(x > 1, 1, 0)",
             write={
@@ -1276,12 +1295,14 @@ class TestDialect(Validator):
     def test_limit(self):
         self.validate_all(
             "SELECT * FROM data LIMIT 10, 20",
-            write={"sqlite": "SELECT * FROM data LIMIT 10 OFFSET 20"},
+            write={"sqlite": "SELECT * FROM data LIMIT 20 OFFSET 10"},
         )
         self.validate_all(
             "SELECT x FROM y LIMIT 10",
             read={
+                "teradata": "SELECT TOP 10 x FROM y",
                 "tsql": "SELECT TOP 10 x FROM y",
+                "snowflake": "SELECT TOP 10 x FROM y",
             },
             write={
                 "sqlite": "SELECT x FROM y LIMIT 10",
@@ -1392,7 +1413,8 @@ class TestDialect(Validator):
                 "presto": "SELECT a AS b FROM x GROUP BY 1",
                 "hive": "SELECT a AS b FROM x GROUP BY 1",
                 "oracle": "SELECT a AS b FROM x GROUP BY 1",
-                "spark": "SELECT a AS b FROM x GROUP BY 1",
+                "spark": "SELECT a AS b FROM x GROUP BY b",
+                "spark2": "SELECT a AS b FROM x GROUP BY 1",
             },
         )
         self.validate_all(
