@@ -141,6 +141,9 @@ A OR C;
 (A OR C) AND (A OR B OR C);
 A OR C;
 
+A AND (B AND C) AND (D AND E);
+A AND B AND C AND D AND E;
+
 --------------------------------------
 -- Elimination
 --------------------------------------
@@ -198,6 +201,21 @@ A AND B AND C AND D;
 (((((A) AND B)) AND C)) AND D;
 A AND B AND C AND D;
 
+(x + 1) + 2;
+x + 3;
+
+x + (1 + 2);
+x + 3;
+
+(x * 2) * 4 + (1 + 3) + 5;
+x * 8 + 9;
+
+(x - 1) - 2;
+(x - 1) - 2;
+
+x - (3 - 2);
+x - 1;
+
 --------------------------------------
 -- Comparison and Pruning
 --------------------------------------
@@ -222,6 +240,12 @@ A AND B AND C;
 SELECT x WHERE TRUE;
 SELECT x;
 
+SELECT x FROM y LEFT JOIN z ON TRUE;
+SELECT x FROM y CROSS JOIN z;
+
+SELECT x FROM y JOIN z USING (x);
+SELECT x FROM y JOIN z USING (x);
+
 --------------------------------------
 -- Parenthesis removal
 --------------------------------------
@@ -249,6 +273,12 @@ x = x;
 (('a' = 'a') AND TRUE and NOT FALSE);
 TRUE;
 
+(x = y) and z;
+x = y AND z;
+
+x * (1 - y);
+x * (1 - y);
+
 --------------------------------------
 -- Literals
 --------------------------------------
@@ -273,6 +303,9 @@ TRUE;
 -1 + 3;
 2;
 
+1 - 2 - 4;
+-5;
+
 -(-1);
 1;
 
@@ -298,16 +331,19 @@ TRUE;
 0.0219;
 
 1 / 3;
-0;
+1 / 3;
+
+1 / 3.0;
+0.3333333333333333333333333333;
 
 20.0 / 6;
 3.333333333333333333333333333;
 
 10 / 5;
-2;
+10 / 5;
 
 (1.0 * 3) * 4 - 2 * (5 / 2);
-8.0;
+12.0 - 2 * (5 / 2);
 
 6 - 2 + 4 * 2 + a;
 12 + a;
@@ -554,3 +590,9 @@ x <= 10 AND x >= 9;
 
 1 < x AND 3 < x;
 x > 3;
+
+'a' < 'b';
+TRUE;
+
+x = 2018 OR x <> 2018;
+x <> 2018 OR x = 2018;
