@@ -1,3 +1,6 @@
+SET x TO 1;
+SET x = 1;
+
 SELECT * FROM test;
 SELECT
   *
@@ -40,7 +43,7 @@ WITH cte1 AS (
         FROM (SELECT 1) AS x, y, (SELECT 2) z
         UNION ALL
         SELECT MAX(COALESCE(x AND y, a and b and c, d and e)), FOO(CASE WHEN a and b THEN c and d ELSE 3 END)
-        GROUP BY x, GROUPING SETS (a, (b, c)) CUBE(y, z)
+        GROUP BY x, GROUPING SETS (a, (b, c)), CUBE(y, z)
     ) x
 )
 SELECT a, b c FROM (
@@ -95,7 +98,7 @@ WITH cte1 AS (
       MAX(COALESCE(x AND y, a AND b AND c, d AND e)),
       FOO(CASE WHEN a AND b THEN c AND d ELSE 3 END)
     GROUP BY
-      x
+      x,
     GROUPING SETS (
       a,
       (b, c)
@@ -175,10 +178,23 @@ FROM (
 ) AS x;
 
 INSERT OVERWRITE TABLE x VALUES (1, 2.0, '3.0'), (4, 5.0, '6.0');
-INSERT OVERWRITE TABLE x VALUES
+INSERT OVERWRITE TABLE x
+VALUES
   (1, 2.0, '3.0'),
   (4, 5.0, '6.0');
 
+INSERT INTO TABLE foo REPLACE WHERE cond SELECT * FROM bar;
+INSERT INTO foo
+REPLACE WHERE cond
+SELECT
+  *
+FROM bar;
+
+INSERT OVERWRITE TABLE zipcodes PARTITION(state = '0') VALUES (896, 'US', 'TAMPA', 33607);
+INSERT OVERWRITE TABLE zipcodes PARTITION(state = '0')
+VALUES
+  (896, 'US', 'TAMPA', 33607);
+
 WITH regional_sales AS (
     SELECT region, SUM(amount) AS total_sales
     FROM orders
@@ -342,3 +358,8 @@ SELECT
   basket_index
 FROM table_data
 CROSS JOIN UNNEST(fruit_basket) AS fruit WITH OFFSET AS basket_index;
+SELECT A.* EXCEPT A.COL_1, A.COL_2 FROM TABLE_1 A;
+SELECT
+  A.*
+  EXCEPT (A.COL_1, A.COL_2)
+FROM TABLE_1 AS A;
